@@ -1,9 +1,62 @@
-(** Relative path with unspecified root *)
-module Relative : sig
-  include Path_intf.S
+(** Representation of paths *)
 
-  (** [root] refers to empty relative path, so whatever the path is interpreted
-      relative to. *)
+(** The aim of this module is to provide a solid basis to reason about
+    file and directory paths inside the Dune code base.  What it is not
+    is a complete API for paths management that handles all the aspects
+    of file system paths.  It simply exposes a high-level and portable
+    API that covers the needs of Dune.
+
+    {1 Model of the file system}
+
+    {2 Local paths}
+
+    Dune sees the file system as two parts. The first part is composed
+    of the source tree and the build directory.  In this part, Dune
+    doesn't know about symlinks and has a fully expanded view of the
+    file system.  This means that if the user has a symlink `src/foo`
+    pointing to `bar`, then `src/foo/x` and `bar/x` are seen as two
+    different paths.
+
+    A path in this world is called a local path and is simply a sequence
+    of path components.  A path component being a string other than "."
+    or ".." and not containing the path separator character ('/').
+
+    Such a path can be rooted at the source tree root, the build directory or
+    an unspecified root.  All these paths are represented by values of type
+    ['a Path.Local_gen.t] where ['a] denotes the root of the path.
+
+    {2 External paths}
+
+    The second part is the "external world".  It is all the paths that live
+    outside of the workspace and build directory.  To be on the safe side
+    Dune makes no assumption does nothing clever with these paths.
+
+    External paths are presented as [Path.External.t] values.contents
+
+    {1 The Path.t type}
+
+    The [Path.t] type represents all possible paths, i.e. both local and
+    extenral paths.
+*)
+
+(** Relative path relative to the root tracked by the type system.
+
+    Represented as: either the root, or a '/' separated list of components
+    other that ".", ".."  and not containing a '/'. *)
+module Local_gen : Path_intf.Local_gen
+
+module Unspecified : sig
+  type w = Path_intf.Unspecified.w
+end
+
+(** Relative path with unspecified root.
+
+    Either root, or a '/' separated list of components
+    other that ".", ".."  and not containing a '/'. *)
+module Local : sig
+  type w = Unspecified.w
+  type t = w Local_gen.t
+  include Path_intf.S with type t := t
   val root : t
 
   module L : sig
@@ -15,32 +68,22 @@ module Relative : sig
   val explode : t -> string list
 end
 
-(** In the current workspace (anything under the current project root) *)
-module Local : sig
-  include Path_intf.S
-  val root : t
-
-  module L : sig
-    val relative : ?error_loc:Loc0.t -> t -> string list -> t
-  end
-
-  val relative : ?error_loc:Loc0.t -> t -> string -> t
-  val split_first_component : t -> (string * Relative.t) option
-  val explode : t -> string list
-end
-
 (** In the source section of the current workspace. *)
 module Source : sig
-  include Path_intf.S
+  type w
+
+  type t = w Local_gen.t
+
+  include Path_intf.S with type t := t
   val root : t
 
   module L : sig
     val relative : ?error_loc:Loc0.t -> t -> string list -> t
   end
 
-  val of_relative : Relative.t -> t
+  val of_local : Local.t -> t
   val relative : ?error_loc:Loc0.t -> t -> string -> t
-  val split_first_component : t -> (string * Relative.t) option
+  val split_first_component : t -> (string * Local.t) option
   val explode : t -> string list
 
   (** [Source.t] does not statically forbid overlap with build directory,
@@ -50,29 +93,6 @@ module Source : sig
   val to_local : t -> Local.t
 end
 
-module Build : sig
-  include Path_intf.S
-  val root : t
-
-  val append_source : t -> Source.t -> t
-
-  val append_relative : t -> Relative.t -> t
-
-  val append_local : t -> Local.t -> t
-
-  module L : sig
-    val relative : ?error_loc:Loc0.t -> t -> string list -> t
-  end
-
-  val relative : ?error_loc:Loc0.t -> t -> string -> t
-  val split_first_component : t -> (string * Relative.t) option
-  val explode : t -> string list
-
-  val drop_build_context     : t -> Source.t option
-  val drop_build_context_exn : t -> Source.t
-end
-
-(** In the outside world *)
 module External : sig
   include Path_intf.S
 
@@ -85,26 +105,73 @@ module External : sig
   val mkdir_p : t -> unit
 end
 
-module Kind : sig
-  type t = private
-    | External of External.t
-    | Local    of Local.t
+module Build : sig
+  type w
 
-  val of_string : string -> t
+  type t = w Local_gen.t
+
+  include Path_intf.S with type t := t
+  val root : t
+
+  val append_source : t -> Source.t -> t
+
+  val append_local : t -> Local.t -> t
+
+  module L : sig
+    val relative : ?error_loc:Loc0.t -> t -> string list -> t
+  end
+
+  val relative : ?error_loc:Loc0.t -> t -> string -> t
+  val split_first_component : t -> (string * Local.t) option
+  val explode : t -> string list
+
+  val local : t -> Local.t
+
+  val drop_build_context     : t -> Source.t option
+  val drop_build_context_exn : t -> Source.t
+
+  (** [Source.t] here is a lie in some cases: consider when the context name
+      happens to be ["install"] or [".alias"]. *)
+  val extract_build_context  : t -> (string * Source.t) option
+  val extract_build_context_exn  : t -> (string * Source.t)
+  val extract_build_context_dir : t -> (t * Source.t) option
+  val extract_build_context_dir_exn : t -> (t * Source.t)
+
+  (** This function does the same as [extract_build_context], but has a
+      "righter" type.  *)
+  val extract_first_component : t -> (string * Local.t) option
+
+  val is_alias_stamp_file : t -> bool
+
+  module Kind : sig
+    type t = private
+      | External of External.t
+      | In_source_dir of Local.t
+
+    val of_string : string -> t
+  end
+
+  (** set the build directory. Can only be called once and must be done before
+      paths are converted to strings elsewhere. *)
+  val set_build_dir : Kind.t -> unit
 end
 
-include Path_intf.S
+type t = private
+  | External of External.t
+  | In_source_tree of Source.t
+  | In_build_dir of Build.t
+
+include Path_intf.S with type t := t
 
 val hash : t -> int
 
-module Table : Hashtbl.S with type key = t
-
 (** [to_string_maybe_quoted t] is [maybe_quoted (to_string t)] *)
 val to_string_maybe_quoted : t -> string
 
-val kind : t -> Kind.t
-
 val root : t
+
+val external_ : External.t -> t
+
 val is_root : t -> bool
 
 val is_managed : t -> bool
@@ -128,7 +195,6 @@ val descendant : t -> of_:t -> t option
 val is_descendant : t -> of_:t -> bool
 
 val append : t -> t -> t
-val append_relative : t -> Relative.t -> t
 val append_local : t -> Local.t -> t
 val append_source : t -> Source.t -> t
 
@@ -146,7 +212,7 @@ val extend_basename : t -> suffix:string -> t
 val extract_build_context     : t -> (string * Source.t) option
 val extract_build_context_exn : t -> (string * Source.t)
 
-val extract_build_dir_first_component     : t -> (string * Relative.t) option
+val extract_build_dir_first_component     : t -> (string * Local.t) option
 
 (** Same as [extract_build_context] but return the build context as a path:
 
@@ -168,10 +234,6 @@ val drop_optional_build_context : t -> t
     otherwise fail. *)
 val drop_optional_build_context_src_exn : t -> Source.t
 
-(** Transform managed paths so that they are descedant of
-    [sandbox_dir]. *)
-val sandbox_managed_paths : sandbox_dir:Build.t -> t -> t
-
 val explode : t -> string list option
 val explode_exn : t -> string list
 
@@ -187,8 +249,6 @@ val as_in_source_tree : t -> Source.t option
 val as_in_build_dir : t -> Build.t option
 val as_in_build_dir_exn : t -> Build.t
 
-val is_alias_stamp_file : t -> bool
-
 (** [is_strict_descendant_of_build_dir t = is_in_build_dir t && t <>
     build_dir] *)
 val is_strict_descendant_of_build_dir : t -> bool
@@ -208,17 +268,12 @@ val unlink_no_err : t -> unit
 val rm_rf : t -> unit
 val mkdir_p : t -> unit
 
-val pp_in_source : Format.formatter -> t -> unit
 val pp_debug : Format.formatter -> t -> unit
 
 val build_dir_exists : unit -> bool
 
 val ensure_build_dir_exists : unit -> unit
 
-(** set the build directory. Can only be called once and must be done before
-    paths are converted to strings elsewhere. *)
-val set_build_dir : Kind.t -> unit
-
 val source : Source.t -> t
 val build : Build.t -> t
 
@@ -231,11 +286,6 @@ val of_local : Local.t -> t
     absolute *)
 val set_root : External.t -> unit
 
-(** Internal use only *)
-module Internal : sig
-  val raw_kind : t -> Kind.t
-end
-
 module L : sig
   val relative : t -> string list -> t
 end
@@ -245,7 +295,7 @@ end
     this returns the path itself.
     For external paths, it returns a path that is relative to the current
     directory. For example, the local part of [/a/b] is [./a/b]. *)
-val local_part : t -> Relative.t
+val local_part : t -> Local.t
 
 val stat : t -> Unix.stats
 
